# This script creates coordinats for piano keys to later use them with makeMIDI.py
#
# Take a screenshot of your video (same resolution) and run this script.
# You may want to exclude some keys below or later edit them in the generated json file.
#
# feel free to use, edit & share
# Written by gilex, 2023 (https://github.com/gilex-dev)
import cv2
import json

notes = {'A""""': {}, 'A#""""': {}, 'H""""': {}, 'C"""': {}, 'C#"""': {}, 'D"""': {}, 'D#"""': {}, 'E"""': {},
         'F"""': {}, 'F#"""': {}, 'G"""': {}, 'G#"""': {}, 'A"""': {}, 'A#"""': {}, 'H"""': {}, 'C""': {}, 'C#""': {},
         'D""': {}, 'D#""': {}, 'E""': {}, 'F""': {}, 'F#""': {}, 'G""': {}, 'G#""': {}, 'A""': {}, 'A#""': {},
         'H""': {}, 'C"': {}, 'C#"': {}, 'D"': {}, 'D#"': {}, 'E"': {}, 'F"': {}, 'F#"': {}, 'G"': {}, 'G#"': {},
         'A"': {}, 'A#"': {}, 'H"': {}, 'C': {}, 'C#': {}, 'D': {}, 'D#': {}, 'E': {}, 'F': {}, 'F#': {}, 'G': {},
         'G#': {}, 'A': {}, 'A#': {}, 'H': {}, 'c': {}, 'c#': {}, 'd': {}, 'd#': {}, 'e': {}, 'f': {}, 'f#': {},
         'g': {}, 'g#': {}, 'a': {}, 'a#': {}, 'h': {}, 'c"': {}, 'c#"': {}, 'd"': {}, 'd#"': {}, 'e"': {}, 'f"': {},
         'f#"': {}, 'g"': {}, 'g#"': {}, 'a"': {}, 'a#"': {}, 'h"': {}, 'c""': {}, 'c#""': {}, 'd""': {}, 'd#""': {},
         'e""': {}, 'f""': {}, 'f#""': {}, 'g""': {}, 'g#""': {}, 'a""': {}, 'a#""': {}, 'h""': {}, 'c"""': {}}
source_img = 'img.png'
progress = 0
pitch_base = 21
start_note = ''
exclude_note = ''
only_white_keys = False
for note in notes.copy():
    pitch = pitch_base + progress
    if '#' in note or note.lower() == 'b':
        if only_white_keys:
            progress += 1
            notes.pop(note)
            continue
        default = 'blackKey'
    else:
        default = 'whiteKey'
    notes[note].update({'pitch': pitch, 'default': default})
    progress += 1
progress = 0
marker_radius = 5


def draw_circle(event, x, y, flags, param):
    global mouseX, mouseY, progress
    # if event == cv2.EVENT_LBUTTONDBLCLK:
    if event == cv2.EVENT_LBUTTONDOWN:
        cv2.circle(img, (x, y), marker_radius, (0, 0, 255), -1)
        mouseX, mouseY = x, y
        notes[list(notes.keys())[progress]].update({'w': x, 'h': y})
        progress += 1
        print(f'Note {progress}/{len(notes)} {int(progress / len(notes) * 100)}%')


img = cv2.imread(source_img)
cv2.namedWindow('image')
cv2.setMouseCallback('image', draw_circle)
while (1):
    cv2.imshow('image', img)
    k = cv2.waitKey(20) & 0xFF
    if k == 27:
        break
    elif k == ord('a'):
        print(mouseX, mouseY)
    if cv2.getWindowProperty('image', cv2.WND_PROP_VISIBLE) < 1:
        break
    if progress >= len(notes):
        print('Done!')
        break
cv2.destroyAllWindows()
print(notes)


def save_profile(output_path, profile):
    with open(output_path, 'w') as output_file:
        output_file.write(json.dumps(profile, indent=4))


save_profile(f'profiles/{input("Profile Name: ")}.json', notes)
